#!/usr/bin/env python
##
##  flvrec.py - VNC to FLV recording tool.
##
##  Copyright (c) 2009 by Yusuke Shinyama
##

import sys, time, socket, os, os.path, subprocess, signal
from vnc2flv.flv import FLVWriter
from vnc2flv.rfb import RFBNetworkClient, RFBError, PWDFile, PWDCache
from vnc2flv.video import FLVVideoSink, str2clip, str2size


##  flvrec
##
def flvrec(filename, host='localhost', port=5900,
           framerate=15, keyframe=150,
           preferred_encoding=(0,), pwdfile=None,
           blocksize=32, clipping=None,
           cmdline=None,
           debug=0):
  print 'recoding to %r...' % filename
  fp = file(filename, 'wb')
  if pwdfile:
    pwdcache = PWDFile(pwdfile)
  else:
    pwdcache = PWDCache('%s:%d' % (host,port))
  writer = FLVWriter(fp, debug=debug)
  sink = FLVVideoSink(writer,
                      blocksize=blocksize, framerate=framerate, keyframe=keyframe,
                      clipping=clipping, debug=debug)
  client = RFBNetworkClient(host, port, sink,
                            pwdcache=pwdcache, preferred_encoding=preferred_encoding,
                            debug=debug)
  subproc = None
  try:
    client.open()
    if debug:
      print >>sys.stderr, 'start recording'
    if cmdline:
      (basename,_) = os.path.splitext(filename)
      subproc = subprocess.Popen(cmdline, shell=True, env={'BASENAME':basename})
    while 1:
      client.idle()
    if debug:
      print >>sys.stderr, 'stop recording'
  except KeyboardInterrupt:
    pass
  except socket.error, e:
    print >>sys.stderr, 'Socket error:', e
  except RFBError, e:
    print >>sys.stderr, 'RFB error:', e
  if subproc:
    os.kill(subproc.pid, signal.SIGINT)
  client.close()
  fp.close()
  return


# main
def main(argv):
  import getopt, vnc2flv
  def usage():
    print argv[0], vnc2flv.__version__
    print ('usage: %s [-d] [-o filename] [-r framerate]'
           ' [-e vnc_encoding] [-P vnc_pwdfile] [-N]'
           ' [-B blocksize] [-C clipping] [-S subprocess]'
           ' [host[:display] [port]]' % argv[0])
    return 100
  try:
    (opts, args) = getopt.getopt(argv[1:], 'do:r:K:e:P:NB:C:S:')
  except getopt.GetoptError:
    return usage()
  debug = 0
  filename = 'out%s.flv' % time.strftime('%Y%m%d%H%M')
  framerate = 15
  keyframe = 150
  preferred_encoding = (0,)
  pwdfile = None
  cursor = True
  blocksize = 32
  clipping = None
  cmdline = None
  (host, port) = ('localhost', 5900)
  for (k, v) in opts:
    if k == '-d': debug += 1
    elif k == '-o': filename = v
    elif k == '-r': framerate = int(v)
    elif k == '-K': keyframe = int(v)
    elif k == '-e': preferred_encoding = tuple( int(i) for i in v.split(',') )
    elif k == '-P': pwdfile = v
    elif k == '-N': cursor = False
    elif k == '-B': blocksize = int(v)
    elif k == '-C': clipping = str2clip(v)
    elif k == '-S': cmdline = v
  if not cursor:
    preferred_encoding += (-232,-239,)
  if 1 <= len(args):
    if ':' in args[0]:
      i = args[0].index(':')
      host = args[0][:i] or 'localhost'
      port = int(args[0][i+1:])+5900
    else:
      host = args[0]
  if 2 <= len(args):
    port = int(args[1])
  flvrec(filename, host, port, framerate=framerate, keyframe=keyframe,
         preferred_encoding=preferred_encoding, pwdfile=pwdfile,
         blocksize=blocksize, clipping=clipping, cmdline=cmdline,
         debug=debug)
  return

if __name__ == "__main__": sys.exit(main(sys.argv))
